/**
 * Created by crismo on 24.01.15.
 */
SuperGame.Game = function(game){

    // Delte konstanter
    SuperGame.TILE_HEIGHT = 32;
    SuperGame.TILE_WIDTH = 32;
    SuperGame.TILE_ROWS = Math.floor(SuperGame.GAME_HEIGHT / SuperGame.TILE_HEIGHT);
    SuperGame.TILE_COLUMNS = Math.floor(SuperGame.GAME_WIDTH / SuperGame.TILE_WIDTH);
    SuperGame.AVATAR_COUNT = 4;

    // Delte variabler
    SuperGame.selectedAvatar = null;

    // Private variabler
    this.walkspeed = 100;
    this.walls = null;
    this.movableWalls = null;
    this.avatars =[];
    this.zero_row = 0;
    this.zero_column = 0;
    this.playtimeRemaining = 0;
    this.fontStyle = null
    this.timerText = null;
    this.exitTile = null;
    this.crashCount = 0;
};

SuperGame.Game.prototype = {
    create: function(){
        gameListener = this;

        this.physics.startSystem(Phaser.Physics.ARCADE);

        this.input.onDown.add(function(){
            this.setAvatarWalking();
        }, this);

        this.zero_row =  Math.round((SuperGame.TILE_ROWS - SuperGame.Map.objects.length)/2);
        this.zero_column = Math.round((SuperGame.TILE_COLUMNS - SuperGame.Map.objects[0].length)/2);

        this.createMapView();
        this.createAvatarsAndObjects();

        this.playtimeRemaining = SuperGame.Map.time * 1000; // transform to ms

        this.fontStyle = { font: "40px Arial", fill: "#FFFFFF", stroke: "#333", strokeThickness: 5, align: "center" };
        this.timerText = this.add.text(120, 20, SuperGame.Map.time + "s", this.fontStyle);



    },
    serverSetAvatarWalking:function(id, inX, inY){

        if(SuperGame.controlableAvatars.indexOf(id) != -1){
            return;
        }

        var avatar = _.find(this.avatars, function(av){ return av.avid == id; });
        if(avatar){
            avatar.body.velocity.y = 0;
            avatar.body.velocity.x = 0;
            avatar.rotation = 0;
            avatar.scale.y = 1;

            var walk = false;
            var dy = avatar.y - inY;
            var dx = avatar.x - inX;

            var absDx = Math.abs(dx);
            var absDy = Math.abs(dy);

            if(absDx < absDy && absDy>= SuperGame.TILE_HEIGHT){
                avatar.body.velocity.y = this.walkspeed * ((dy > 0) ? -1:1);
                avatar.targetX = Math.floor(avatar.x) ;
                avatar.targetY = Math.floor(this.boundToRow(inY) + 16);
                avatar.scale.y = ((dy > 0) ? 1:-1);
                walk = true;
            }

            if(absDx > absDy  && absDx >= SuperGame.TILE_WIDTH){
                avatar.body.velocity.x = this.walkspeed * ((dx > 0) ? -1:1);
                avatar.targetX = Math.floor(this.boundToColumn(inX) + 16);
                avatar.targetY = Math.floor(avatar.y);
                var angle = (dx > 0) ? -90:90;
                angle = angle*Math.PI/180;
                avatar.rotation = angle;
                walk = true;
            }


            if(walk){
                avatar.isMoving = true;
                avatar.play("walk");
            }

        }
    },
    setAvatarWalking:function(){

        if(SuperGame.selectedAvatar !== null){

            SuperGame.selectedAvatar.body.velocity.y = 0;
            SuperGame.selectedAvatar.body.velocity.x = 0;
            SuperGame.selectedAvatar.rotation = 0;
            SuperGame.selectedAvatar.scale.y = 1;

            var walk = false;
            var dy = SuperGame.selectedAvatar.y - game.input.y;
            var dx = SuperGame.selectedAvatar.x - game.input.x;

            var absDx = Math.abs(dx);
            var absDy = Math.abs(dy);

            if(absDx < absDy && absDy>= SuperGame.TILE_HEIGHT){
                SuperGame.selectedAvatar.body.velocity.y = this.walkspeed * ((dy > 0) ? -1:1);
                SuperGame.selectedAvatar.targetX = Math.floor(SuperGame.selectedAvatar.x) ;//this.boundToColumn(SuperGame.selectedAvatar.x);
                SuperGame.selectedAvatar.targetY = Math.floor(this.boundToRow(game.input.y) + 16);
                SuperGame.selectedAvatar.scale.y = ((dy > 0) ? 1:-1);
                walk = true;
            }

            if(absDx > absDy  && absDx >= SuperGame.TILE_WIDTH){
                SuperGame.selectedAvatar.body.velocity.x = this.walkspeed * ((dx > 0) ? -1:1);
                SuperGame.selectedAvatar.targetX = Math.floor(this.boundToColumn(game.input.x) + 16);
                SuperGame.selectedAvatar.targetY = Math.floor(SuperGame.selectedAvatar.y);
                var angle = (dx > 0) ? -90:90;
                angle = angle*Math.PI/180;
                SuperGame.selectedAvatar.rotation = angle;
                walk = true;
            }

            console.log(SuperGame.selectedAvatar.targetX + "  " + SuperGame.selectedAvatar.targetY);

           if(walk){
               sendMove( SuperGame.selectedAvatar.avid,  SuperGame.selectedAvatar.targetX,  SuperGame.selectedAvatar.targetY);
               SuperGame.selectedAvatar.isMoving = true;
               SuperGame.selectedAvatar.play("walk");
               SuperGame.selectedAvatar = null;
            }
        }

    },
    createAvatarsAndObjects:function() {

        this.movableWalls = game.add.group();
        this.movableWalls.enableBody = true;
        this.movableWalls.physicsBodyType = Phaser.Physics.ARCADE;

        for(var row = 0; row <  SuperGame.Map.objects.length; row++) {
            for (var column = 0; column <  SuperGame.Map.objects[row].length; column++) {
                var value = SuperGame.Map.objects[row][column];

                var tx = Math.floor((column + this.zero_column) * SuperGame.TILE_WIDTH) ;
                var ty = Math.floor((row + this.zero_row) * SuperGame.TILE_HEIGHT);

                if (value >= 1 && value <= 4){
                    var av = SuperGame.Avatar.create(game, row + this.zero_row, column + this.zero_column, 'avatar_' + value);
                    av.avid = value;
                    this.avatars.push(av);
                } else if( value == TileInfo.MOVABLE_WALL_TILE){


                    //tileIndex bassert på om hjørne?
                    //var tileIndex = TileInfo.randomMovableWallTile();
                    var tileIndex = 1;

                    var tileName = "movableWallTiles";

                    var sp = this.movableWalls.create(tx,ty,tileName );

                    sp.body.bounce.set(0,0);
                    sp.body.immovable = false;
                    sp.body.width = 28;
                    sp.body.height = 28;

                } else if( value == TileInfo.EXIT_TILE){

                    this.exitTile = game.add.sprite(tx, ty, "exitTile");
                    this.exitTile.animations.add('magic',[0,1,2], 10, false);
                    this.exitTile.animations.play('idle');

                    game.physics.enable(this.exitTile, Phaser.Physics.ARCADE);
                    this.exitTile.body.bounce.set(0,0);
                    this.exitTile.body.immovable = true;
                    this.exitTile.body.width = 28;
                    this.exitTile.body.height = 28;

                }
            }
        }
    },
    createMapView:function(){

        this.walls = game.add.group();
        this.walls.enableBody = true;
        this.walls.physicsBodyType = Phaser.Physics.ARCADE;

        for(var row = 0; row <  SuperGame.Map.walls.length; row++) {
            for (var collumn = 0; collumn < SuperGame.Map.walls[row].length; collumn++) {
                game.add.sprite(xPos, yPos, tileName,tileIndex);
            }
        }


        for(var row = 0; row <  SuperGame.Map.walls.length; row++) {
            for (var collumn = 0; collumn <  SuperGame.Map.walls[row].length; collumn++) {
                var value = SuperGame.Map.walls[row][collumn];
                var drawRow = row + this.zero_row
                var drawCollumn = collumn + this.zero_column;
                var tileIndex = 0;
                var tileName = "";
                var usePhysics = 0;

                // TODO: Alle disse if / else greiene kan erstattes med en assosiative array / dictionary.
                if (value == TileInfo.FLOOR_TILE) {
                    tileIndex = TileInfo.randomFloorTile();
                    tileName = "floorTiles";
                //} else if( value == TileInfo.UNMOVABLE_WALL_TILE){
                } else if( value >= 20 && value < 28){
                    tileIndex = value - 20;
                    tileName = "unmovableWallTiles";
                    usePhysics = 1;
                }

                /*
                if(value >= 25 && value < 28){
                    //skal ha gulv i tillegg

                    console.log("Gulvtile under!");

                    game.add.sprite(xPos, yPos, "floorTiles",TileInfo.randomFloorTile());
                }
                */

                var xPos = drawCollumn * SuperGame.TILE_WIDTH;
                var yPos = drawRow * SuperGame.TILE_HEIGHT;

                if(usePhysics == 1){
                    var sp = this.walls.create(xPos, yPos, tileName, tileIndex);
                    sp.body.width = 30;
                    sp.body.height = 30;
                    sp.body.bounce.set(0,0);
                    sp.body.immovable = true;
                }else{
                    game.add.sprite(xPos, yPos, tileName,tileIndex);
                }
            }
        }
    },
    update: function(){

        this.playtimeRemaining -= this.time.elapsed;
        this.timerText.setText( Math.floor(this.playtimeRemaining / 1000) + "s" );
        if(this.playtimeRemaining <= 0){
            ///TODO: Tror virkelig vi trenger en gamover state med en horibel slutt beskjed.
            SuperGame.current_levelIndex++;
            this.state.start("Menu");
        }

        for(var i = 0; i< this.avatars.length; i ++){
            if(this.avatars[i].isMoving){
                var distance = game.physics.arcade.distanceToXY(this.avatars[i],this.avatars[i].targetX,this.avatars[i].targetY);
                if(distance < 2){
                    var row = this.calculateRow(this.avatars[i].targetY);
                    var column = this.calculateColumn(this.avatars[i].targetX);
                    this.avatars[i].y = ((row + this.zero_row ) * SuperGame.TILE_HEIGHT) + 16;
                    this.avatars[i].x = ((column + this.zero_column ) * SuperGame.TILE_WIDTH)+16;
                    this.avatarColide(this.avatars[i],null);
                } else{
                    game.physics.arcade.collide(this.avatars[i], this.walls, this.avatarColide, null, this);
                    game.physics.arcade.collide(this.avatars[i], this.movableWalls, this.avatarColide, null, this);
                    game.physics.arcade.collide(this.avatars[i], this.exitTile, this.exit, null, this);


                }
            }
        }

        game.physics.arcade.collide(this.movableWalls,this.walls, this.movableWallsColide, null, this);
        game.physics.arcade.collide(this.movableWalls,this.movableWalls,this.movableWallsColide, null, this);
    },

    exit:function(avatar,exit){
        this.crashCount ++;
        avatar.visible = false;

        var index = this.avatars.indexOf(avatar);
        this.avatars.splice(index, 1);

        avatar.x = -50;
        avatar.y = -50;
        avatar.visible = false;


        if(this.crashCount == 4){
            SuperGame.current_levelIndex++;
            this.state.start("LoadLevel");
        }
    },

    // Collision -------------------------------------------------------------------------------------------------------

    movableWallsColide: function(wall1,wall2){
        wall2.body.velocity.x = wall1.body.velocity.x = 0;
        wall2.body.velocity.y = wall1.body.velocity.y = 0;

        wall1.x = this.boundToColumn(wall1.x+16);
        wall1.y = this.boundToRow(wall1.y+16);

        wall2.x = this.boundToColumn(wall2.x+16);
        wall2.y = this.boundToRow(wall2.y+16);
    },

    avatarColide: function(avatar,wall) {
         avatar.isMoving = false;
         avatar.body.velocity.x = 0;
         avatar.body.velocity.y = 0;
         avatar.play("idle");
    },

    // UTILITY FUNCTIONS -----------------------------------------------------------------------------------------------
    calculateRow: function(ypos){
        return Math.floor(ypos / SuperGame.TILE_HEIGHT) - this.zero_row;
    },
    calculateColumn: function(xpos){
        return Math.floor(xpos / SuperGame.TILE_WIDTH) - this.zero_column;
    },
    boundToRow:function(ypos){
        return ( Math.floor(ypos / SuperGame.TILE_HEIGHT)  * SuperGame.TILE_HEIGHT);
    },
    boundToColumn:function(xpos){
        return ( Math.floor(xpos / SuperGame.TILE_WIDTH) * SuperGame.TILE_WIDTH);
    }
};

SuperGame.Avatar = {
    create:function(game,row,collumn, avatarSprite){
        var avatar = game.add.sprite(0, 0, avatarSprite);
        avatar.anchor.setTo(0.5, 0.5);
        avatar.x = Math.floor(collumn * SuperGame.TILE_WIDTH) + 16;
        avatar.y = Math.floor(row * SuperGame.TILE_HEIGHT) + 16;

        avatar.inputEnabled = true;
        avatar.events.onInputDown.add(this.onClicked, this);

        avatar.animations.add('idle',[0], 10, false);
        avatar.animations.add('walk',[1,2], 10, true);
        avatar.animations.play('idle');

        game.physics.enable(avatar, Phaser.Physics.ARCADE);
        avatar.body.bounce.set(0,0);
        avatar.body.immovable = false;
        avatar.body.width = 28;
        avatar.body.height = 28;

        return avatar;
    },
    onClicked:function(avatar){
        if(SuperGame.controlableAvatars.indexOf(avatar.avid) != -1){
            avatar.yTarget = null;
            avatar.xTarget = null;
            SuperGame.selectedAvatar = avatar;
        }
    }
};
