/**
 * Created by crismo on 24.01.15.
 */

var _ = require('underscore');
var express = require("express");
var app = express();
var port = 8080;
var http = require('http').Server(app);
var io = require('socket.io')(http);


var MAX_PLAYERS = 4
var playerCount = 0;
var isOpen = true;
var players = [];
var startLevelCount = 0;


app.use(express.static(__dirname));

io.on('connection', function(socket){

    if(playerCount >= MAX_PLAYERS){
        isOpen = false;
    }

    if(isOpen){
        playerCount ++;
        players.push(socket);
        notifyNewPlayerCountChanged();
        console.log('player connected, online now ' + playerCount);
    } else{
        console.warn('No rom for more players ' + playerCount);
        socket.disconnect();
    }

    socket.on("isOpen",function(){
        console.log("isOpen request");
        var index = players.indexOf(socket);
        players[index].emit("isOpen", {"value":isOpen, "first":playerCount == 1});
    });


    socket.on("start", function(){
        var avatarID = _.shuffle([1,2,3,4]);
        var avatarsPerPlayer = Math.floor(4 / playerCount);
        console.log("Avatars per player" + avatarsPerPlayer);
        for(var i = 1; i < players.length; i++){
            players[i].emit("gameStart",{"avatars": avatarID.splice(0,avatarsPerPlayer)});
        }
        players[0].emit("gameStart",{"avatars": avatarID});
    });

    socket.on("startLevel", function(){
        startLevelCount++;
        console.log("Start level " + startLevelCount);
        if(startLevelCount == playerCount){
            console.log("Start level GO!!" );
            io.emit("startLevel",{});
            startLevelCount = 0;
        }
    });

    socket.on("move", function(data){
       io.emit("avatarMove",data);
    });

    socket.on("disconnect",function() {
        var index = players.indexOf(socket);
        if (index != -1) {
            playerCount --;
            players.splice(index, 1);
            console.info('Player gone (id=' + socket.id + ').');
            notifyNewPlayerCountChanged();
            if(index == 0 && playerCount > 0){
                players[0].emit("isOpen", {"value":isOpen, "first":true});
            }
        }

    });

});

function notifyNewPlayerCountChanged(){
    console.log("Notify peeps");
    io.emit("newPlayer",{"numberOfPlayers":playerCount});
}

http.listen(port, function(){
    console.log("Listening on port " + port);
});

